@@ -1,12 +1,12 @@
 /*
- *   $Id: privsep-bsd44.c,v 1.1 2008/01/24 10:03:17 psavola Exp $
+ *   $Id: privsep-bsd44.c,v 1.2 2010/12/14 11:58:21 psavola Exp $
  *
  *   Authors:
  *    Jim Paris			<jim@jtan.com>
  *    Pedro Roque		<roque@di.fc.ul.pt>
- *    Lars Fenneberg		<lf@elemental.net>	 
+ *    Lars Fenneberg		<lf@elemental.net>
  *
- *   This software is Copyright 1996,1997,2008 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997,2008 by the above mentioned author(s),
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
@@ -15,10 +15,10 @@
  *
  */
 
-#include <config.h>
-#include <includes.h>
-#include <radvd.h>
-#include <pathnames.h>
+#include "config.h"
+#include "includes.h"
+#include "radvd.h"
+#include "pathnames.h"
 
 /* Not available */
 int
