@@ -1,5 +1,5 @@
 /*
- *   $Id: process.c,v 1.21 2010/12/14 11:58:21 psavola Exp $
+ *   $Id: process.c,v 1.26 2011/04/04 14:24:58 reubenhwk Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
@@ -18,7 +18,7 @@
 #include "includes.h"
 #include "radvd.h"
 
-static void process_rs(int, struct Interface *, unsigned char *msg,
+static void process_rs(struct Interface *, unsigned char *msg,
 		       int len, struct sockaddr_in6 *);
 static void process_ra(struct Interface *, unsigned char *msg, int len,
 	struct sockaddr_in6 *);
@@ -26,16 +26,18 @@ static int  addr_match(struct in6_addr *a1, struct in6_addr *a2,
 	int prefixlen);
 
 void
-process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
+process(struct Interface *ifacel, unsigned char *msg, int len,
 	struct sockaddr_in6 *addr, struct in6_pktinfo *pkt_info, int hoplimit)
 {
 	struct Interface *iface;
 	struct icmp6_hdr *icmph;
 	char addr_str[INET6_ADDRSTRLEN];
 
+	print_addr(&addr->sin6_addr, addr_str);
+
 	if ( ! pkt_info )
 	{
-		flog(LOG_WARNING, "received packet with no pkt_info!" );
+		flog(LOG_WARNING, "received packet with no pkt_info from %s!", addr_str );
 		return;
 	}
 
@@ -45,8 +47,8 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 
 	if (len < sizeof(struct icmp6_hdr))
 	{
-		flog(LOG_WARNING, "received icmpv6 packet with invalid length: %d",
-			len);
+		flog(LOG_WARNING, "received icmpv6 packet with invalid length (%d) from %s",
+			len, addr_str);
 		return;
 	}
 
@@ -66,13 +68,13 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 	if (icmph->icmp6_type == ND_ROUTER_ADVERT)
 	{
 		if (len < sizeof(struct nd_router_advert)) {
-			flog(LOG_WARNING, "received icmpv6 RA packet with invalid length: %d",
-				len);
+			flog(LOG_WARNING, "received icmpv6 RA packet with invalid length (%d) from %s",
+				len, addr_str);
 			return;
 		}
 
 		if (!IN6_IS_ADDR_LINKLOCAL(&addr->sin6_addr)) {
-			flog(LOG_WARNING, "received icmpv6 RA packet with non-linklocal source address");
+			flog(LOG_WARNING, "received icmpv6 RA packet with non-linklocal source address from %s", addr_str);
 			return;
 		}
 	}
@@ -80,16 +82,16 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 	if (icmph->icmp6_type == ND_ROUTER_SOLICIT)
 	{
 		if (len < sizeof(struct nd_router_solicit)) {
-			flog(LOG_WARNING, "received icmpv6 RS packet with invalid length: %d",
-				len);
+			flog(LOG_WARNING, "received icmpv6 RS packet with invalid length (%d) from %s",
+				len, addr_str);
 			return;
 		}
 	}
 
 	if (icmph->icmp6_code != 0)
 	{
-		flog(LOG_WARNING, "received icmpv6 RS/RA packet with invalid code: %d",
-			icmph->icmp6_code);
+		flog(LOG_WARNING, "received icmpv6 RS/RA packet with invalid code (%d) from %s",
+			icmph->icmp6_code, addr_str);
 		return;
 	}
 
@@ -130,16 +132,18 @@ process(int sock, struct Interface *ifacel, unsigned char *msg, int len,
 
 	if (icmph->icmp6_type == ND_ROUTER_SOLICIT)
 	{
-		process_rs(sock, iface, msg, len, addr);
+		flog(LOG_WARNING, "received RS from %s", addr_str);
+		process_rs(iface, msg, len, addr);
 	}
 	else if (icmph->icmp6_type == ND_ROUTER_ADVERT)
 	{
+		flog(LOG_WARNING, "received RA from %s", addr_str);
 		process_ra(iface, msg, len, addr);
 	}
 }
 
 static void
-process_rs(int sock, struct Interface *iface, unsigned char *msg, int len,
+process_rs(struct Interface *iface, unsigned char *msg, int len,
 	struct sockaddr_in6 *addr)
 {
 	double delay;
@@ -187,28 +191,24 @@ process_rs(int sock, struct Interface *iface, unsigned char *msg, int len,
 
 	gettimeofday(&tv, NULL);
 
-	delay = MAX_RA_DELAY_TIME*rand()/(RAND_MAX+1.0);
-	dlog(LOG_DEBUG, 3, "random mdelay for %s: %.2f", iface->Name, delay);
+	delay = MAX_RA_DELAY_TIME * rand() / (RAND_MAX +1.0);
 
 	if (iface->UnicastOnly) {
+		dlog(LOG_DEBUG, 3, "random mdelay for %s: %g seconds.", iface->Name, delay/1000.0);
 		mdelay(delay);
-		send_ra_forall(sock, iface, &addr->sin6_addr);
+		send_ra_forall(iface, &addr->sin6_addr);
 	}
-	else if ((tv.tv_sec + tv.tv_usec / 1000000.0) - (iface->last_multicast_sec +
-	          iface->last_multicast_usec / 1000000.0) < iface->MinDelayBetweenRAs) {
-		/* last RA was sent only a few moments ago, don't send another immediately */
-		clear_timer(&iface->tm);
-		next = iface->MinDelayBetweenRAs - (tv.tv_sec + tv.tv_usec / 1000000.0) +
-		       (iface->last_multicast_sec + iface->last_multicast_usec / 1000000.0) + delay/1000.0;
-		set_timer(&iface->tm, next);
+	else if ( timevaldiff(&tv, &iface->last_multicast) / 1000.0 < iface->MinDelayBetweenRAs ) {
+		/* last RA was sent only a few moments ago, don't send another immediately. */
+		dlog(LOG_DEBUG, 3, "random mdelay for %s: %g seconds.", iface->Name, delay/1000.0);
+		next = iface->MinDelayBetweenRAs - (tv.tv_sec + tv.tv_usec / 1000000.0) + (iface->last_multicast.tv_sec + iface->last_multicast.tv_usec / 1000000.0) + delay/1000.0;
+		iface->next_multicast = next_timeval(next);
 	}
 	else {
-		/* no RA sent in a while, send an immediate multicast reply */
-		clear_timer(&iface->tm);
-		if (send_ra_forall(sock, iface, NULL) == 0) {
-			next = rand_between(iface->MinRtrAdvInterval, iface->MaxRtrAdvInterval);
-			set_timer(&iface->tm, next);
-		}
+		/* no RA sent in a while, send a multicast reply */
+		send_ra_forall(iface, NULL);
+		next = rand_between(iface->MinRtrAdvInterval, iface->MaxRtrAdvInterval);
+		iface->next_multicast = next_timeval(next);
 	}
 }
 
@@ -333,7 +333,7 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 				{
 					print_addr(&prefix->Prefix, prefix_str);
 
-					if (valid != prefix->AdvValidLifetime)
+					if (!prefix->DecrementLifetimesFlag && valid != prefix->AdvValidLifetime)
 					{
 						flog(LOG_WARNING, "our AdvValidLifetime on"
 						 " %s for %s doesn't agree with %s",
@@ -342,7 +342,7 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 						 addr_str
 						 );
 					}
-					if (preferred != prefix->AdvPreferredLifetime)
+					if (!prefix->DecrementLifetimesFlag && preferred != prefix->AdvPreferredLifetime)
 					{
 						flog(LOG_WARNING, "our AdvPreferredLifetime on"
 						 " %s for %s doesn't agree with %s",
@@ -414,6 +414,7 @@ process_ra(struct Interface *iface, unsigned char *msg, int len,
 			break;
 		case ND_OPT_DNSSL_INFORMATION:
 			dnsslinfo = (struct nd_opt_dnssl_info_local *) opt_str;
+			suffix[0] = '\0';
 			for (offset = 0; offset < (dnsslinfo->nd_opt_dnssli_len-1)*8;) {
 				label_len = dnsslinfo->nd_opt_dnssli_suffixes[offset++];
 
