@@ -1,5 +1,5 @@
 /*
- *   $Id: send.c,v 1.38 2011/01/07 17:26:27 reubenhwk Exp $
+ *   $Id: send.c,v 1.48 2011/05/06 07:51:54 reubenhwk Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
@@ -27,13 +27,13 @@
  *
  */
 int
-send_ra_forall(int sock, struct Interface *iface, struct in6_addr *dest)
+send_ra_forall(struct Interface *iface, struct in6_addr *dest)
 {
 	struct Clients *current;
 
 	/* If no list of clients was specified for this interface, we broadcast */
 	if (iface->ClientList == NULL)
-		return send_ra(sock, iface, dest);
+		return send_ra(iface, dest);
 
 	/* If clients are configured, send the advertisement to all of them via unicast */
 	for (current = iface->ClientList; current; current = current->next)
@@ -47,7 +47,7 @@ send_ra_forall(int sock, struct Interface *iface, struct in6_addr *dest)
 		if (dest != NULL && memcmp(dest, &current->Address, sizeof(struct in6_addr)) != 0)
 			continue;
 		dlog(LOG_DEBUG, 5, "Sending RA to %s", address_text);
-		send_ra(sock, iface, &(current->Address));
+		send_ra(iface, &(current->Address));
 
 		/* If we should only send the RA to a specific address, we are done */
 		if (dest != NULL)
@@ -77,8 +77,42 @@ send_ra_inc_len(size_t *len, int add)
 	}
 }
 
+static time_t
+time_diff_secs(const struct timeval *time_x, const struct timeval *time_y)
+{
+	time_t secs_diff;
+
+	secs_diff = time_x->tv_sec - time_y->tv_sec;
+	if ((time_x->tv_usec - time_y->tv_usec) >= 500000)
+		secs_diff++;
+
+	return secs_diff;
+	
+}
+
+static void
+decrement_lifetime(const time_t secs, uint32_t *lifetime)
+{
+
+	if (*lifetime > secs) {
+		*lifetime -= secs;	
+	} else {
+		*lifetime = 0;
+	}
+}
+
+static void cease_adv_pfx_msg(const char *if_name, struct in6_addr *pfx, const int pfx_len)
+{
+	char pfx_str[INET6_ADDRSTRLEN];
+
+	print_addr(pfx, pfx_str);
+
+	dlog(LOG_DEBUG, 3, "Will cease advertising %s/%u%%%s, preferred lifetime is 0", pfx_str, pfx_len, if_name);
+
+}
+
 int
-send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
+send_ra(struct Interface *iface, struct in6_addr *dest)
 {
 	uint8_t all_hosts_addr[] = {0xff,0x02,0,0,0,0,0,0,0,0,0,0,0,0,0,1};
 	struct sockaddr_in6 addr;
@@ -92,13 +126,15 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 	struct AdvRoute *route;
 	struct AdvRDNSS *rdnss;
 	struct AdvDNSSL *dnssl;
+	struct timeval time_now;
+	time_t secs_since_last_ra;
 
 	unsigned char buff[MSG_SIZE_SEND];
 	size_t len = 0;
 	ssize_t err;
 
 	/* First we need to check that the interface hasn't been removed or deactivated */
-	if(check_device(sock, iface) < 0) {
+	if(check_device(iface) < 0) {
 		if (iface->IgnoreIfMissing)  /* a bit more quiet warning message.. */
 			dlog(LOG_DEBUG, 4, "interface %s does not exist, ignoring the interface", iface->Name);
 		else {
@@ -123,22 +159,25 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 	}
 
 	/* Make sure that we've joined the all-routers multicast group */
-	if (check_allrouters_membership(sock, iface) < 0)
+	if (check_allrouters_membership(iface) < 0)
 		flog(LOG_WARNING, "problem checking all-routers membership on %s", iface->Name);
 
 	dlog(LOG_DEBUG, 3, "sending RA on %s", iface->Name);
 
 	if (dest == NULL)
 	{
-		struct timeval tv;
-
 		dest = (struct in6_addr *)all_hosts_addr;
-		gettimeofday(&tv, NULL);
-
-		iface->last_multicast_sec = tv.tv_sec;
-		iface->last_multicast_usec = tv.tv_usec;
+		gettimeofday(&iface->last_multicast, NULL);
 	}
 
+	gettimeofday(&time_now, NULL);
+	secs_since_last_ra = time_diff_secs(&time_now, &iface->last_ra_time);
+	if (secs_since_last_ra < 0) {
+		secs_since_last_ra = 0;
+		flog(LOG_WARNING, "gettimeofday() went backwards!");
+	}
+	iface->last_ra_time = time_now;
+
 	memset((void *)&addr, 0, sizeof(addr));
 	addr.sin6_family = AF_INET6;
 	addr.sin6_port = htons(IPPROTO_ICMPV6);
@@ -160,8 +199,12 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 	radvert->nd_ra_flags_reserved   |=
 		(iface->AdvHomeAgentFlag)?ND_RA_FLAG_HOME_AGENT:0;
 
-	/* if forwarding is disabled, send zero router lifetime */
-	radvert->nd_ra_router_lifetime	 =  !check_ip6_forwarding() ? htons(iface->AdvDefaultLifetime) : 0;
+	if (iface->cease_adv) {
+		radvert->nd_ra_router_lifetime = 0;
+	} else {
+		/* if forwarding is disabled, send zero router lifetime */
+		radvert->nd_ra_router_lifetime	 =  !check_ip6_forwarding() ? htons(iface->AdvDefaultLifetime) : 0;
+	}
 	radvert->nd_ra_flags_reserved   |=
 		(iface->AdvDefaultPreference << ND_OPT_RI_PRF_SHIFT) & ND_OPT_RI_PRF_MASK;
 
@@ -178,7 +221,7 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 
 	while(prefix)
 	{
-		if( prefix->enabled )
+		if( prefix->enabled && prefix->curr_preferredlft > 0 )
 		{
 			struct nd_opt_prefix_info *pinfo;
 
@@ -196,8 +239,24 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 			pinfo->nd_opt_pi_flags_reserved |=
 				(prefix->AdvRouterAddr)?ND_OPT_PI_FLAG_RADDR:0;
 
-			pinfo->nd_opt_pi_valid_time	= htonl(prefix->AdvValidLifetime);
-			pinfo->nd_opt_pi_preferred_time = htonl(prefix->AdvPreferredLifetime);
+			if (iface->cease_adv && prefix->DeprecatePrefixFlag) {
+				/* RFC4862, 5.5.3, step e) */
+				pinfo->nd_opt_pi_valid_time	= htonl(MIN_AdvValidLifetime);
+				pinfo->nd_opt_pi_preferred_time = 0;
+			} else {
+				if (prefix->DecrementLifetimesFlag) {
+					decrement_lifetime(secs_since_last_ra,
+								&prefix->curr_validlft);
+					
+					decrement_lifetime(secs_since_last_ra,
+								&prefix->curr_preferredlft);
+					if (prefix->curr_preferredlft == 0)
+						cease_adv_pfx_msg(iface->Name, &prefix->Prefix, prefix->PrefixLen);
+				}
+				pinfo->nd_opt_pi_valid_time	= htonl(prefix->curr_validlft);
+				pinfo->nd_opt_pi_preferred_time = htonl(prefix->curr_preferredlft);
+
+			}
 			pinfo->nd_opt_pi_reserved2	= 0;
 
 			memcpy(&pinfo->nd_opt_pi_prefix, &prefix->Prefix,
@@ -228,7 +287,11 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 
 		rinfo->nd_opt_ri_flags_reserved  =
 			(route->AdvRoutePreference << ND_OPT_RI_PRF_SHIFT) & ND_OPT_RI_PRF_MASK;
-		rinfo->nd_opt_ri_lifetime	= htonl(route->AdvRouteLifetime);
+		if (iface->cease_adv && route->RemoveRouteFlag) {
+			rinfo->nd_opt_ri_lifetime	= 0;
+		} else {
+			rinfo->nd_opt_ri_lifetime	= htonl(route->AdvRouteLifetime);
+		}
 
 		memcpy(&rinfo->nd_opt_ri_prefix, &route->Prefix,
 		       sizeof(struct in6_addr));
@@ -253,7 +316,11 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 		rdnssinfo->nd_opt_rdnssi_len	     = 1 + 2*rdnss->AdvRDNSSNumber;
 		rdnssinfo->nd_opt_rdnssi_pref_flag_reserved = 0;
 
-		rdnssinfo->nd_opt_rdnssi_lifetime	= htonl(rdnss->AdvRDNSSLifetime);
+		if (iface->cease_adv && rdnss->FlushRDNSSFlag) {
+			rdnssinfo->nd_opt_rdnssi_lifetime	= 0;
+		} else {
+			rdnssinfo->nd_opt_rdnssi_lifetime	= htonl(rdnss->AdvRDNSSLifetime);
+		}
 
 		memcpy(&rdnssinfo->nd_opt_rdnssi_addr1, &rdnss->AdvRDNSSAddr1,
 		       sizeof(struct in6_addr));
@@ -284,7 +351,11 @@ send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
 		dnsslinfo->nd_opt_dnssli_len 		= 1; /* more further down */
 		dnsslinfo->nd_opt_dnssli_reserved	= 0;
 
-		dnsslinfo->nd_opt_dnssli_lifetime	= htonl(dnssl->AdvDNSSLLifetime);
+		if (iface->cease_adv && dnssl->FlushDNSSLFlag) {
+			dnsslinfo->nd_opt_dnssli_lifetime	= 0;
+		} else {
+			dnsslinfo->nd_opt_dnssli_lifetime	= htonl(dnssl->AdvDNSSLLifetime);
+		}
 
 		buff_ptr = dnsslinfo->nd_opt_dnssli_suffixes;
 		for (i = 0; i < dnssl->AdvDNSSLNumber; i++) {
